@@ -80,21 +80,21 @@ sub input_filter {
         my $filter = $choices->[$idx];
         my $filter_str = sprintf( "Filter: %s", $filter );
         if ( $filter eq $reset ) {
+            $sf->__print_filter_info( $sql, 3, undef );
             $sql->{insert_into_args} = [ map { [ @$_ ] } @{$sf->{i}{gc}{bu_insert_into_args}} ];
             $sf->{empty_to_null} = $default_e2n;
             next FILTER
         }
         elsif ( $filter eq $confirm ) {
             if ( $sf->{empty_to_null} ) {
+                $sf->__print_filter_info( $sql, 3, undef );
                 no warnings 'uninitialized';
                 $sql->{insert_into_args} = [ map { [ map { length ? $_ : undef } @$_ ] } @{$sql->{insert_into_args}} ];
             }
-            delete $sf->{i}{gc}{working};
             return 1;
         }
         elsif ( $filter eq $reparse ) {
             $sf->__print_filter_info( $sql, 7, undef );
-            delete $sf->{i}{gc}{working};
             return -1;
         }
         elsif ( $filter eq $choose_cols  ) {
@@ -150,6 +150,7 @@ sub input_filter {
 sub __print_filter_info {
     my ( $sf, $sql, $row_count, $horizontal_choices ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    print $sf->{i}{gc}{working} . "\r";
     $sf->{i}{occupied_term_height} = 1; #
     #$sf->{i}{occupied_term_height} += 1; # keep bottom line empty
     $sf->{i}{occupied_term_height} += $row_count;
@@ -207,7 +208,7 @@ sub __choose_columns {
     my $col_idx = $tu->choose_a_subset(
         $header,
         { current_selection_label => 'Cols: ', layout => 0, order => 0, mark => $mark, all_by_default => 1,
-          index => 1, confirm => $sf->{i}{ok}, back => '<<', info => $filter_str } # order
+          index => 1, confirm => $sf->{i}{ok}, back => '<<', info => $filter_str }
     );
     if ( ! defined $col_idx ) {
         return;
@@ -221,22 +222,29 @@ sub __choose_rows {
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $aoa = $sql->{insert_into_args};
+    $sf->__print_filter_info( $sql, 2 + @$aoa, undef );
     my @pre = ( undef, $sf->{i}{ok} );
     my @stringified_rows;
+    my $mark;
     {
         no warnings 'uninitialized';
-        @stringified_rows = map { join ',', @$_ } @$aoa;
+        for my $i ( 0 .. $#$aoa ) {
+            push @$mark, $i + @pre if length join '', @{$aoa->[$i]};
+            push @stringified_rows, join ',', @{$aoa->[$i]};
+        }
+    }
+    if ( @$mark == @stringified_rows ) {
+        $mark = undef;
     }
     my $prompt = 'Choose rows:';
-    $sf->__print_filter_info( $sql, 2 + @$aoa, undef ); # before resetting $sql->{insert_into_args}
     $sql->{insert_into_args} = []; # $sql->{insert_into_args} refers to a new new empty array - this doesn't delete $aoa
 
     while ( 1 ) {
         # Choose
         my @idx = $tc->choose(
             [ @pre, @stringified_rows ],
-            { %{$sf->{i}{lyt_v}}, prompt => $prompt, info => $filter_str, meta_items => [ 0 .. $#pre ], include_highlighted => 2,
-                index => 1, undef => '<<', busy_string => $sf->{i}{gc}{working} }
+            { %{$sf->{i}{lyt_v}}, prompt => $prompt, info => $filter_str, meta_items => [ 0 .. $#pre ],
+              include_highlighted => 2, index => 1, undef => '<<', busy_string => $sf->{i}{gc}{working}, mark => $mark }
         );
         $sf->__print_filter_info( $sql, 2 + @$aoa, undef );
         if ( ! $idx[0] ) {
@@ -655,6 +663,7 @@ sub __merge_rows {
     my $tu = Term::Choose::Util->new( $sf->{i}{tcu_default} );
     my $tf = Term::Form->new( $sf->{i}{tf_default} );
     my $aoa = $sql->{insert_into_args};
+    $sf->__print_filter_info( $sql, 2 + @$aoa, undef );
     my $term_w = get_term_width();
     my @stringified_rows;
     {
@@ -669,7 +678,6 @@ sub __merge_rows {
             }
         } @$aoa;
     }
-    $sf->__print_filter_info( $sql, 2 + @$aoa, undef );
     my $prompt = 'Choose rows:';
     my $chosen_idxs = $tu->choose_a_subset(
         \@stringified_rows,
@@ -680,6 +688,7 @@ sub __merge_rows {
     if ( ! defined $chosen_idxs || ! @$chosen_idxs ) {
         return;
     }
+    $sf->__print_filter_info( $sql, 2 + @{$aoa->[$chosen_idxs->[0]]}, undef );
     my $merged = [];
     for my $col ( 0 .. $#{$aoa->[$chosen_idxs->[0]]} ) {
         my @tmp;
@@ -693,7 +702,6 @@ sub __merge_rows {
     }
     my $col_number = 0;
     my $fields = [ map { [ ++$col_number, defined $_ ? "$_" : '' ] } @$merged ];
-    $sf->__print_filter_info( $sql, 2 + @$fields, undef );
     # Fill_form
     my $form = $tf->fill_form(
         $fields,
@@ -745,6 +753,7 @@ sub __join_columns {
     if ( ! defined $join_char ) {
         return;
     }
+    $sf->__print_filter_info( $sql, 2 + @$aoa, undef );
     my $merged = [];
     for my $row ( 0 .. $#{$aoa} ) {
         my @tmp;
@@ -758,7 +767,6 @@ sub __join_columns {
     }
     my $col_number = 0;
     my $fields = [ map { [ ++$col_number, defined $_ ? "$_" : '' ] } @$merged ];
-    $sf->__print_filter_info( $sql, 2 + @$fields, undef );
     # Fill_form
     my $form = $tf->fill_form(
         $fields,
@@ -774,7 +782,7 @@ sub __join_columns {
     for my $row ( 0 .. $#{$aoa} ) { # modifies $aoa
         $aoa->[$row][$first_idx] = $merged->[$row];
         for my $idx ( sort { $b <=> $a } @$chosen_idxs ) {
-            splice @{$aoa->[$row]}, $idx, 1;
+            splice @{$aoa->[$row]}, $idx, 1 if $idx < @{$aoa->[$row]};
         }
     }
     $sql->{insert_into_args} = $aoa;
